@@ -4,7 +4,8 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 from collections import deque
-import os
+from os import environ
+from os.path import isfile
 
 from sapphire.server_map import ServerMap
 from .storage import TestCase, TestFile
@@ -22,9 +23,10 @@ class IOManager(object):
         "LSAN_OPTIONS",
         "GNOME_ACCESSIBILITY",
         "GRZ_FORCED_CLOSE",
-        "MOZ_CHAOSMODE")
+        "MOZ_CHAOSMODE",
+        "XPCOM_DEBUG_BREAK")
 
-    def __init__(self, report_size=1, mime_type=None, working_path=None):
+    def __init__(self, report_size=1, working_path=None):
         assert report_size > 0
         self.harness = None
         self.server_map = ServerMap()  # manage redirects, include directories and dynamic responses
@@ -32,7 +34,6 @@ class IOManager(object):
         self.working_path = working_path
         self._environ_files = list()  # collection of files that should be added to the testcase
         self._generated = 0  # number of test cases generated
-        self._mime = mime_type
         self._report_size = report_size
         # used to record environment variable that directly impact the browser
         self._tracked_env = self.tracked_environ()
@@ -46,18 +47,16 @@ class IOManager(object):
 
     def _add_suppressions(self):
         # Add suppression files to environment files
-        for env_var in (e_var for e_var in os.environ if "SAN_OPTIONS" in e_var):
-            opts = sanitizer_opts(os.environ.get(env_var, ""))
+        for env_var in (x for x in environ if "SAN_OPTIONS" in x):
+            opts = sanitizer_opts(environ.get(env_var, ""))
             if "suppressions" not in opts:
                 continue
             supp_file = opts["suppressions"].strip("'\"")
-            if os.path.isfile(supp_file):
+            if isfile(supp_file):
                 fname = "%s.supp" % (env_var.split("_")[0].lower(),)
                 self._environ_files.append(TestFile.from_file(supp_file, fname))
 
     def cleanup(self):
-        if self.harness is not None:
-            self.harness.close()
         for e_file in self._environ_files:
             e_file.close()
         self.purge_tests()
@@ -76,12 +75,14 @@ class IOManager(object):
             test.add_meta(e_file.clone())
         # reset redirect map
         self.server_map.redirect.clear()
+        self.server_map.set_redirect("grz_current_test", self.page_name(), required=False)
+        self.server_map.set_redirect("grz_next_test", self.page_name(offset=1))
         if self.harness is not None:
-            # setup redirects for harness
-            self.server_map.set_redirect("first_test", self.page_name(), required=False)
-            self.server_map.set_redirect("next_test", self.page_name(offset=1))
             # add harness to testcase
-            test.add_file(self.harness.clone(), required=False)
+            self.server_map.set_dynamic_response(
+                "grz_harness",
+                lambda: self.harness,
+                mime_type="text/html")
         self._generated += 1
         self.tests.append(test)
         # manage testcase cache size
@@ -89,11 +90,6 @@ class IOManager(object):
             self.tests.popleft().cleanup()
         return test
 
-    def landing_page(self):
-        if self.harness is None:
-            return self.page_name()
-        return self.harness.file_name
-
     def page_name(self, offset=0):
         return "test_%04d.html" % (self._generated + offset,)
 
@@ -108,11 +104,11 @@ class IOManager(object):
         # that are relevant to Grizzly or the test case.
         env = dict()
         tracked_san_opts = ("detect_leaks",)
-        for e_var in IOManager.TRACKED_ENVVARS:
-            if e_var not in os.environ:
+        for var in IOManager.TRACKED_ENVVARS:
+            if var not in environ:
                 continue
-            if e_var.endswith("SAN_OPTIONS"):
-                opts = sanitizer_opts(os.environ.get(e_var, ""))
+            if var.endswith("SAN_OPTIONS"):
+                opts = sanitizer_opts(environ.get(var, ""))
                 # strip unwanted options
                 tracked = dict()
                 for opt in tracked_san_opts:
@@ -120,7 +116,10 @@ class IOManager(object):
                         tracked[opt] = opts[opt]
                 # only record *SAN_OPTIONS if there are options
                 if tracked:
-                    env[e_var] = ":".join("=".join((k, v)) for k, v in tracked.items())
+                    env[var] = ":".join("=".join((k, v)) for k, v in tracked.items())
+            elif var == "XPCOM_DEBUG_BREAK" and environ.get(var, "").lower() == "warn":
+                # ignore FFPuppet default XPCOM_DEBUG_BREAK value (set in helpers.py)
+                continue
             else:
-                env[e_var] = os.environ[e_var]
+                env[var] = environ[var]
         return env
