@@ -2,8 +2,8 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-import logging
-import time
+from logging import getLogger
+from time import sleep, time
 
 from sapphire import SERVED_TIMEOUT
 from ..target import TargetLaunchTimeout
@@ -12,7 +12,7 @@ __all__ = ("Runner",)
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith"]
 
-LOG = logging.getLogger("grz_runner")
+LOG = getLogger("grz_runner")
 
 # _IdleChecker is used to help determine if the target is hung (actively using CPU)
 # or if it has not made expected the HTTP requests for other reasons (idle).
@@ -45,7 +45,7 @@ class _IdleChecker(object):
             bool: True if the target idle callback returned True otherwise False
         """
         assert self._next_poll is not None, "schedule_poll() must be called first"
-        now = time.time()
+        now = time()
         if now >= self._next_poll:
             if self._check_cb(self._threshold):
                 return True
@@ -63,7 +63,7 @@ class _IdleChecker(object):
             None
         """
         if now is None:
-            now = time.time()
+            now = time()
         if initial:
             self._next_poll = now + self._init_delay
         else:
@@ -108,16 +108,16 @@ class Runner(object):
         for retries in reversed(range(max_retries)):
             try:
                 self._target.launch(location, env_mod=env_mod)
-                break
             except TargetLaunchTimeout:
                 # likely has nothing to do with Grizzly but is seen frequently
                 # on machines under a high load. After multiple consecutive timeouts
                 # something is likely wrong so raise.
                 if retries:
                     LOG.warning("Launch timeout (attempts remaining %d)", retries)
-                    time.sleep(retry_delay)
+                    sleep(retry_delay)
                     continue
                 raise
+            break
 
     @staticmethod
     def location(srv_path, srv_port, close_after=None, forced_close=True, timeout=None):
@@ -133,7 +133,7 @@ class Runner(object):
         Returns:
             str: A valid URL.
         """
-        location = "http://127.0.0.1:%d/%s" % (srv_port, srv_path)
+        location = "http://127.0.0.1:%d/%s" % (srv_port, srv_path.lstrip("/"))
         # set harness related arguments
         args = []
         if close_after is not None:
@@ -148,13 +148,14 @@ class Runner(object):
             return "?".join([location, "&".join(args)])
         return location
 
-    def run(self, ignore, server_map, testcase, wait_for_callback=False):
+    def run(self, ignore, server_map, testcase, coverage=False, wait_for_callback=False):
         """Serve a testcase and monitor the target for results.
 
         Args:
             ignore (list): List of failure types to ignore.
             server_map (sapphire.ServerMap): A ServerMap.
-            testcase: (grizzly.TestCase): The test case that will be served.
+            testcase (grizzly.TestCase): The test case that will be served.
+            coverage (bool): Trigger coverage dump.
             wait_for_callback: (bool): Use `_keep_waiting()` to indicate when
                                        framework should move on.
 
@@ -173,16 +174,25 @@ class Runner(object):
             continue_cb=self._keep_waiting,
             forever=wait_for_callback,
             server_map=server_map)
+        # add all include files that were served
+        for url, resource in server_map.include.items():
+            testcase.add_batch(resource.target, self.served, prefix=url)
         self.timeout = server_status == SERVED_TIMEOUT
-        # detect failure
-        failure_detected = self._target.detect_failure(ignore, self.timeout)
         served_lpage = testcase.landing_page in self.served
         if not served_lpage:
-            # output message even if a failure was detected
             LOG.debug("%r not served!", testcase.landing_page)
+        elif coverage and not self.timeout:
+            # dump_coverage() should be called before detect_failure()
+            # to help catch any coverage related issues.
+            self._target.dump_coverage()
+        # detect failure
+        failure_detected = self._target.detect_failure(ignore, self.timeout)
         if failure_detected == self._target.RESULT_FAILURE:
             self.result = self.FAILED
         elif not served_lpage:
+            # something is wrong so close the target
+            # previous iteration put target in a bad state?
+            self._target.close()
             self.result = self.ERROR
         elif failure_detected == self._target.RESULT_IGNORED:
             self.result = self.IGNORED
@@ -200,5 +210,6 @@ class Runner(object):
             bool: Continue to serve test test case
         """
         if self._idle is not None and self._idle.is_idle():
+            LOG.debug("idle target detected")
             return False
         return self._target.monitor.is_healthy()
