@@ -10,8 +10,6 @@ import os
 import shutil
 import tempfile
 
-import six
-
 from ..target import sanitizer_opts
 
 __all__ = ("TestCase", "TestFile", "TestCaseLoadFailure", "TestFileExists")
@@ -70,6 +68,29 @@ class TestCase(object):
         self._existing_paths.append(test_file.file_name)
         target.append(test_file)
 
+    def add_batch(self, path, include_files, prefix=None):
+        """Iterate over files in include_files and attach the files that are
+        located in path to testcase.
+
+        Args:
+            path (str): Path to the root of the directory that contains files.
+            include_files (iterable): Paths of the files to be added to the
+                                      test case if they exist in path.
+            prefix (str): Path prefix to prepend to file when adding to
+                          test case.
+
+        Returns:
+            None
+        """
+        path = os.path.abspath(path)
+        for fname in (x for x in include_files if x.startswith(path)):
+            test_path = os.path.relpath(fname, path)
+            if test_path.startswith(".."):
+                continue
+            if prefix:
+                test_path = "/".join((prefix, test_path))
+            self.add_from_file(fname, file_name=test_path)
+
     def add_meta(self, meta_file):
         """Add a test file to test case as a meta file.
 
@@ -216,7 +237,7 @@ class TestCase(object):
     def load_environ(self, path, env_data):
         # sanity check environment variable data
         for name, value in env_data.items():
-            if not isinstance(name, six.string_types) or not isinstance(value, six.string_types):
+            if not isinstance(name, str) or not isinstance(value, str):
                 raise TestCaseLoadFailure("'env_data' contains invalid 'env' entries")
         self.env_vars = env_data
         known_suppressions = ("lsan.supp", "tsan.supp", "ubsan.supp")
@@ -272,21 +293,28 @@ class TestCase(object):
         test = cls(None, None, adapter)
         if full_scan:
             # load all files from directory as test
-            for root, _, files in os.walk(path):
+            for dpath, _, files in os.walk(path):
                 for fname in files:
                     if fname == "test_info.json":
                         continue
-                    if root == path:
+                    if dpath == path:
                         if fname == "prefs.js":
                             if prefs:
-                                test.add_meta(TestFile.from_file(os.path.join(path, fname)))
+                                test.add_meta(TestFile.from_file(os.path.join(dpath, fname)))
                             continue
                         if fname == entry_point:
-                            test.add_from_file(os.path.join(root, fname))
+                            test.add_from_file(os.path.join(dpath, fname))
                             # set entry point
                             test.landing_page = fname
                             continue
-                    test.add_from_file(os.path.join(path, fname), required=False)
+                        location = None
+                    else:
+                        # handle nested directories
+                        location = "/".join((dpath.split(path, 1)[-1], fname))
+                    test.add_from_file(
+                        os.path.join(dpath, fname),
+                        file_name=location,
+                        required=False)
         else:
             # load single file as test
             test.add_from_file(os.path.join(path, entry_point))
