@@ -3,7 +3,9 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # pylint: disable=protected-access
-import pytest
+from os.path import join as pathjoin
+
+from pytest import raises
 
 from sapphire import Sapphire, SERVED_ALL, SERVED_NONE, SERVED_REQUEST, SERVED_TIMEOUT, ServerMap
 
@@ -30,12 +32,16 @@ def test_runner_01(mocker):
     assert runner.result == runner.COMPLETE
     assert runner.served == serv_files
     assert not runner.timeout
+    assert target.close.call_count == 0
+    assert target.dump_coverage.call_count == 0
     # some files served
     server.serve_testcase.return_value = (SERVED_REQUEST, serv_files)
-    runner.run([], ServerMap(), testcase)
+    runner.run([], ServerMap(), testcase, coverage=True)
     assert runner.result == runner.COMPLETE
     assert runner.served == serv_files
     assert not runner.timeout
+    assert target.close.call_count == 0
+    assert target.dump_coverage.call_count == 1
 
 def test_runner_02(mocker):
     """test Runner() errors"""
@@ -50,12 +56,14 @@ def test_runner_02(mocker):
     assert runner.result == runner.ERROR
     assert not runner.served
     assert not runner.timeout
+    assert target.close.call_count == 1
+    target.reset_mock()
     # landing page not served
     server.serve_testcase.return_value = (SERVED_REQUEST, ["harness"])
-    target.detect_failure.return_value = target.RESULT_NONE
     runner.run([], ServerMap(), testcase)
     assert runner.result == runner.ERROR
     assert runner.served
+    assert target.close.call_count == 1
 
 def test_runner_03(mocker):
     """test reporting timeout"""
@@ -109,6 +117,7 @@ def test_runner_05(mocker):
     assert runner._idle is not None
     runner.run([], ServerMap(), mocker.Mock(spec=TestCase, landing_page=serv_files[0]))
     assert runner.result == runner.COMPLETE
+    assert target.close.call_count == 0
 
 def test_runner_06(mocker):
     """test Runner._keep_waiting()"""
@@ -139,6 +148,8 @@ def test_runner_07():
     """test Runner.location()"""
     result = Runner.location("a.html", 34567)
     assert result == "http://127.0.0.1:34567/a.html"
+    result = Runner.location("/a.html", 34567)
+    assert result == "http://127.0.0.1:34567/a.html"
     result = Runner.location("a.html", 34567, close_after=10)
     assert result == "http://127.0.0.1:34567/a.html?close_after=10"
     result = Runner.location("a.html", 34567, close_after=10, forced_close=False)
@@ -159,16 +170,47 @@ def test_runner_08(mocker):
     target.reset_mock()
 
     target.launch.side_effect = TargetLaunchError
-    with pytest.raises(TargetLaunchError):
+    with raises(TargetLaunchError):
         runner.launch("http://a/")
     assert target.launch.call_count == 1
     target.reset_mock()
 
     target.launch.side_effect = TargetLaunchTimeout
-    with pytest.raises(TargetLaunchTimeout):
+    with raises(TargetLaunchTimeout):
         runner.launch("http://a/", max_retries=3)
     assert target.launch.call_count == 3
 
+def test_runner_09(mocker, tmp_path):
+    """test Runner.run() adding includes to testcase"""
+    server = mocker.Mock(spec=Sapphire)
+    target = mocker.Mock(spec=Target)
+    target.detect_failure.return_value = target.RESULT_NONE
+    runner = Runner(server, target)
+    # create test files
+    inc_path1 = (tmp_path / "include")
+    inc_path1.mkdir()
+    inc1 = (inc_path1 / "inc_file.bin")
+    inc1.write_bytes(b"a")
+    (inc_path1 / "nested").mkdir()
+    inc2 = (inc_path1 /  "nested" / "nested_inc.bin")
+    inc2.write_bytes(b"a")
+    inc_path2 = (tmp_path / "include2")
+    inc_path2.mkdir()
+    inc3 = (inc_path2 / "inc_file3.txt")
+    inc3.write_bytes(b"a")
+    # build server map
+    smap = ServerMap()
+    smap.set_include("/", str(inc_path1))
+    smap.set_include("/test", str(inc_path2))
+    serv_files = ["a.b", str(inc1), str(inc2), str(inc3)]
+    server.serve_testcase.return_value = (SERVED_ALL, serv_files)
+    with TestCase("a.b", "x", "x") as tcase:
+        runner.run([], smap, tcase)
+        assert runner.result == runner.COMPLETE
+        assert "inc_file.bin" in tcase._existing_paths
+        assert pathjoin("nested", "nested_inc.bin") in tcase._existing_paths
+        assert pathjoin("test", "inc_file3.txt") in tcase._existing_paths
+
 def test_idle_check_01(mocker):
     """test simple _IdleChecker"""
     fake_time = mocker.patch("grizzly.common.runner.time", autospec=True)
@@ -177,7 +219,7 @@ def test_idle_check_01(mocker):
     assert ichk._init_delay == 10
     assert ichk._poll_delay == 1
     assert ichk._next_poll is None
-    fake_time.time.return_value = 0
+    fake_time.return_value = 0
     ichk.schedule_poll(initial=True)
     assert ichk._next_poll == 10
     ichk.schedule_poll()
@@ -190,18 +232,18 @@ def test_idle_check_02(mocker):
     callbk.return_value = False
     #check_cb, delay, duration, threshold
     ichk = _IdleChecker(callbk, 100, 10, poll_delay=1)
-    fake_time.time.return_value = 0
+    fake_time.return_value = 0
     ichk.schedule_poll()
     # early check
     assert not ichk.is_idle()
     assert callbk.call_count == 0
     # not idle
-    fake_time.time.return_value = 10
+    fake_time.return_value = 10
     assert not ichk.is_idle()
     assert ichk._next_poll == 11
     assert callbk.call_count == 1
     # idle
     callbk.return_value = True
-    fake_time.time.return_value = ichk._next_poll
+    fake_time.return_value = ichk._next_poll
     assert ichk.is_idle()
     assert callbk.call_count == 2
