@@ -15,6 +15,7 @@ import os
 import re
 import shutil
 import tempfile
+import time
 import zipfile
 import zlib
 
@@ -24,9 +25,12 @@ from FTB.Signatures.CrashInfo import CrashSignature
 
 from . import strategies as strategies_module, testcase_contents
 from .exceptions import CorruptTestcaseError, NoTestcaseError, ReducerError
+from ..common.reporter import FilesystemReporter, FuzzManagerReporter, Report
+from ..common.runner import Runner
+from ..common.status import ReducerStats, Status
+from ..common.storage import TestCase, TestFile
+from ..main import configure_logging
 from ..session import Session
-from ..common import FilesystemReporter, FuzzManagerReporter, ReducerStats, \
-    Report, Runner, Status, TestCase, TestFile
 from ..target import load as load_target, sanitizer_opts, TargetLaunchError, \
     TargetLaunchTimeout
 
@@ -614,13 +618,8 @@ class ReductionJob(object):
                 harness = os.path.join(os.path.dirname(__file__), '..', 'common', 'harness.html')
                 with open(harness, 'rb') as harness_fp:
                     harness = harness_fp.read()
-
-                def _dyn_resp_close():  # pragma: no cover
-                    self._target.close()
-                    return b"<h1>Close Browser</h1>"
-                self._server_map.set_dynamic_response("/close_browser", _dyn_resp_close, mime_type="text/html")
-                self._server_map.set_dynamic_response("/harness", lambda: harness, mime_type="text/html")
-                self._server_map.set_redirect("/first_test", str(self.landing_page), required=True)
+                self._server_map.set_dynamic_response("grz_harness", lambda: harness, mime_type="text/html")
+            self._server_map.set_redirect("grz_current_test", str(self.landing_page), required=False)
 
         runner = Runner(self._server, self._target, self._idle_threshold, self._idle_timeout)
         if self._no_harness:
@@ -632,24 +631,29 @@ class ReductionJob(object):
         # (re)launch Target
         if self._target.closed:
             if self._no_harness:
-                location = runner.location(self.landing_page, self._server.port)
+                location = runner.location(
+                    "/grz_current_test",
+                    self._server.port)
             else:
                 location = runner.location(
-                    "harness",
+                    "/grz_harness",
                     self._server.port,
                     close_after=self._target.rl_reset,
                     forced_close=self._target.forced_close,
                     timeout=self._iter_timeout)
-            # Try to launch the browser at most, 4 times
-            runner.launch(
-                location,
-                env_mod=self._env_mod,
-                max_retries=4,
-                retry_delay=15)
+            # Try to launch the browser, retry 4 times at most
+            runner.launch(location, env_mod=self._env_mod, max_retries=4, retry_delay=15)
             self._target.step()
 
         if not self._no_harness:
-            self._server_map.set_redirect("/next_test", str(self.landing_page), required=True)
+            def _dyn_resp_close():  # pragma: no cover
+                if self.target.monitor.is_healthy():
+                    # delay to help catch window close/shutdown related crashes
+                    time.sleep(0.1)
+                    self.target.close()
+                return b"<h1>Close Browser</h1>"
+            self._server_map.set_dynamic_response("grz_close_browser", _dyn_resp_close, mime_type="text/html")
+            self._server_map.set_redirect("grz_next_test", str(self.landing_page), required=True)
 
         # run test case
         runner.run(self._ignore, self._server_map, testcase, wait_for_callback=self._no_harness)
@@ -663,7 +667,7 @@ class ReductionJob(object):
             result_logs = temp_prefix + "_logs"
             if not os.path.exists(result_logs):
                 os.mkdir(result_logs)
-            self._target.save_logs(result_logs, meta=True)
+            self._target.save_logs(result_logs)
 
             # create a CrashInfo
             crash = FuzzManagerReporter.create_crash_info(
@@ -846,18 +850,12 @@ class ReductionJob(object):
                 LOG.warning("Using prefs included in testcase: %r", self._target.prefs)
             if "test_info.json" in os.listdir(dirs[0]):
                 self.config_environ(os.path.join(dirs[0], "test_info.json"))
-                if self._env_mod:
-                    LOG.warning("Using environment included in testcase: %s",
-                                os.path.abspath(os.path.join(dirs[0], "test_info.json")))
-                    self._target.forced_close = \
-                        self._env_mod.get("GRZ_FORCED_CLOSE", "1").lower() not in ("0", "false")
             elif "env_vars.txt" in os.listdir(dirs[0]):
                 # TODO: remove this block once move to 'test_info.json' is complete
                 self.config_environ(os.path.join(dirs[0], "env_vars.txt"))
-                LOG.warning("Using environment included in testcase: %s",
-                            os.path.abspath(os.path.join(dirs[0], "env_vars.txt")))
-                self._target.forced_close = \
-                    self._env_mod.get("GRZ_FORCED_CLOSE", "1").lower() not in ("0", "false")
+            if self._env_mod:
+                LOG.warning("Using environment included in testcase")
+                self._target.forced_close = self._env_mod.get("GRZ_FORCED_CLOSE") != "0"
 
             # if dirs is singular, we can use the testcase directly, otherwise we need to iterate over
             # them all in order
@@ -885,8 +883,8 @@ class ReductionJob(object):
                                        "to match harness!")
                 harness = new_harness
                 # make first test and next test grab from the array
-                harness = harness.replace("'/first_test'", "_reduce_next()")
-                harness = harness.replace("'/next_test'", "_reduce_next()")
+                harness = harness.replace("'/grz_current_test'", "_reduce_next()")
+                harness = harness.replace("'/grz_next_test'", "_reduce_next()")
                 # insert the close condition. we are iterating over the array of landing pages,
                 # undefined means we hit the end and the harness should close
                 # newer harness uses conditional operator in open() call
@@ -1159,16 +1157,13 @@ class ReductionJob(object):
             LOG.info("Using a fixed iteration timeout")
         job._fixed_timeout = args.fixed_timeout
 
-        # detect soft assertions
-        if args.soft_asserts:
-            job.target.add_abort_token("###!!! ASSERTION:")
-
         return job
 
     @classmethod
     def main(cls, args):
         # NOTE: this mirrors grizzly.core.main pretty closely
         #       please check if updates here should go there too
+        configure_logging(args.log_level)
         LOG.info("Starting Grizzly Reducer")
         if args.fuzzmanager:
             FuzzManagerReporter.sanity_check(args.binary)
