@@ -51,11 +51,7 @@ def fake_sapphire(monkeypatch):
     monkeypatch.setattr(sapphire, "Sapphire", FakeServer)
     FakeServer._last_timeout = None
 
-@pytest.fixture
-def fake_timesleep(monkeypatch):
-    monkeypatch.setattr(time, "sleep", lambda x: None)
-
-pytestmark = pytest.mark.usefixtures("fake_sapphire", "fake_timesleep")
+pytestmark = pytest.mark.usefixtures("fake_sapphire")
 
 
 def test_interesting(tmp_path):
@@ -67,6 +63,10 @@ def test_interesting(tmp_path):
     obj.lithium_init()
     (tmp_path / "lithium").mkdir()
     assert obj.lithium_interesting(str(tmp_path / "lithium"))
+    assert "grz_close_browser" in obj._server_map.dynamic
+    assert "grz_harness" in obj._server_map.dynamic
+    assert "grz_current_test" in obj._server_map.redirect
+    assert "grz_next_test" in obj._server_map.redirect
     assert obj.target._calls["close"] == 1
     obj.lithium_cleanup()
     assert obj.target._calls["cleanup"] == 0
@@ -145,9 +145,10 @@ def test_target_relaunch_error(tmp_path):
     assert obj.target._calls["cleanup"] == 0
 
 
-def test_target_relaunch_timeout(tmp_path):
+def test_target_relaunch_timeout(mocker, tmp_path):
     "target should be launched more than once on TargetLaunchTimeout"
 
+    mocker.patch("grizzly.common.runner.sleep", autospec=True)
     class MyTarget(FakeTarget):
 
         def launch(self, *args, **kwds):
@@ -180,6 +181,10 @@ def test_no_harness(tmp_path):
     obj.reduce_file = str(reduce_file)
     obj.lithium_init()
     assert obj.lithium_interesting(str(prefix))
+    assert "grz_close_browser" not in obj._server_map.dynamic
+    assert "grz_harness" not in obj._server_map.dynamic
+    assert "grz_current_test" in obj._server_map.redirect
+    assert "grz_next_test" not in obj._server_map.redirect
     assert obj.target._calls["close"] == 1
     obj.lithium_cleanup()
     assert obj.target._calls["cleanup"] == 0
