@@ -7,7 +7,7 @@ from logging import getLogger
 from os.path import isdir
 from shutil import rmtree
 from tempfile import mkdtemp
-from time import time
+from time import sleep, time
 
 from .common import Report, Runner, Status, TestFile
 from .target import TargetLaunchError
@@ -108,7 +108,7 @@ class Session(object):
     def report_result(self):
         # create working directory for target logs
         result_logs = mkdtemp(prefix="grz_logs_", dir=self.iomanager.working_path)
-        self.target.save_logs(result_logs, meta=True)
+        self.target.save_logs(result_logs)
         report = Report.from_path(result_logs)
         crash_info = report.crash_info(self.target.binary)
         short_sig = crash_info.createShortSignature()
@@ -124,10 +124,13 @@ class Session(object):
         runner = Runner(self.server, self.target)
 
         def _dyn_close():  # pragma: no cover
-            self.target.close()
+            if self.target.monitor.is_healthy():
+                # delay to help catch window close/shutdown related crashes
+                sleep(0.1)
+                self.target.close()
             return b"<h1>Close Browser</h1>"
         self.iomanager.server_map.set_dynamic_response(
-            "/close_browser",
+            "grz_close_browser",
             _dyn_close,
             mime_type="text/html")
 
@@ -140,10 +143,14 @@ class Session(object):
                 self.iomanager.purge_tests()
                 self.adapter.pre_launch()
                 if self.iomanager.harness is None:
-                    location = runner.location(self.iomanager.landing_page(), self.server.port)
-                else:
+                    # harness is not in use, open the test case
                     location = runner.location(
-                        self.iomanager.landing_page(),
+                        "/grz_current_test",
+                        self.server.port)
+                else:
+                    # harness is in use, open it and it will open the test case
+                    location = runner.location(
+                        "/grz_harness",
                         self.server.port,
                         close_after=self.target.rl_reset,
                         forced_close=self.target.forced_close,
@@ -165,7 +172,7 @@ class Session(object):
             self.display_status(log_limiter=log_limiter)
 
             # run test case
-            runner.run(ignore, self.iomanager.server_map, current_test)
+            runner.run(ignore, self.iomanager.server_map, current_test, coverage=self.coverage)
             # adapter callbacks
             if runner.timeout:
                 log.debug("calling self.adapter.on_timeout()")
@@ -198,8 +205,6 @@ class Session(object):
                     # since this is the first few iteration something is
                     # likely wrong with the Target (caching?) or the Adapter
                     raise SessionError("Please check Adapter and Target")
-                # relaunching the target to get things back on track
-                self.target.close()
 
             # trigger relaunch by closing the browser if needed
             self.target.check_relaunch()
@@ -213,10 +218,6 @@ class Session(object):
                 log.info("Hit iteration limit")
                 break
 
-            if self.coverage and not self.target.closed:
-                # trigger coverage dump since the target was not closed
-                self.target.dump_coverage()
-
             # warn about large browser logs
             self.status.log_size = self.target.log_size()
             if self.status.log_size > self.TARGET_LOG_SIZE_WARN:
