@@ -2,15 +2,19 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-import logging
-import os
-import platform
+from logging import getLogger
+from os import close, getenv, kill, makedirs, unlink
+from os.path import isdir, isfile
+from platform import system
 import signal
-import time
+from time import localtime, sleep, strftime, time
+from tempfile import mkdtemp, mkstemp
 
-import psutil
+from psutil import AccessDenied, NoSuchProcess, Process, process_iter
 
 from ffpuppet import BrowserTimeoutError, FFPuppet, LaunchError
+from prefpicker import PrefPicker
+
 from .target_monitor import TargetMonitor
 from .target import Target, TargetLaunchError, TargetLaunchTimeout, TargetError
 
@@ -18,11 +22,11 @@ __all__ = ("PuppetTarget",)
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 
-log = logging.getLogger("grizzly")  # pylint: disable=invalid-name
+LOG = getLogger("puppet_target")
 
 
 class PuppetTarget(Target):
-    __slots__ = ("use_rr", "use_valgrind", "_puppet")
+    __slots__ = ("use_rr", "use_valgrind", "_browser_logs", "_puppet", "_tmp_prefs")
 
     def __init__(self, binary, extension, launch_timeout, log_limit, memory_limit, prefs, relaunch, **kwds):
         super(PuppetTarget, self).__init__(binary, extension, launch_timeout, log_limit,
@@ -31,8 +35,23 @@ class PuppetTarget(Target):
         self.use_valgrind = kwds.pop("valgrind", False)
         use_xvfb = kwds.pop("xvfb", False)
         if kwds:
-            log.warning("PuppetTarget ignoring unsupported arguments: %s", ", ".join(kwds))
-
+            LOG.warning("PuppetTarget ignoring unsupported arguments: %s", ", ".join(kwds))
+        self._browser_logs = None
+        # generate prefs.js file if needed
+        if self.prefs is None:
+            for prefs_template in PrefPicker.templates():
+                if prefs_template.endswith("browser-fuzzing.yml"):
+                    LOG.debug("using prefpicker template %r", prefs_template)
+                    tmp_fd, self.prefs = mkstemp(prefix="prefs_", suffix=".js")
+                    close(tmp_fd)
+                    PrefPicker.load_template(prefs_template).create_prefsjs(self.prefs)
+                    LOG.debug("generated prefs.js %r", self.prefs)
+                    break
+            else:  # pragma: no cover
+                raise TargetError("Failed to generate prefs.js")
+            self._tmp_prefs = True
+        else:
+            self._tmp_prefs = False
         # create Puppet object
         self._puppet = FFPuppet(
             use_rr=self.use_rr,
@@ -44,22 +63,36 @@ class PuppetTarget(Target):
         with self._lock:
             proc_usage = self._puppet.cpu_usage()
         for pid, cpu in sorted(proc_usage, reverse=True, key=lambda x: x[1]):
-            log.debug("sending SIGABRT to pid: %r, cpu: %0.2f%%", pid, cpu)
-            os.kill(pid, signal.SIGABRT)
+            LOG.debug("sending SIGABRT to pid: %r, cpu: %0.2f%%", pid, cpu)
+            kill(pid, signal.SIGABRT)
             break
 
     def add_abort_token(self, token):
         self._puppet.add_abort_token(token)
 
     def cleanup(self):
-        # prevent parallel calls to FFPuppet.clean_up()
+        # prevent parallel calls to FFPuppet.close() and/or FFPuppet.clean_up()
+        if self._browser_logs:
+            self.close()
         with self._lock:
             self._puppet.clean_up()
+        if self._tmp_prefs and isfile(self.prefs):
+            unlink(self.prefs)
 
     def close(self):
-        # prevent parallel calls to FFPuppet.close()
+        # prevent parallel calls to FFPuppet.close() and/or FFPuppet.clean_up()
         with self._lock:
             self._puppet.close()
+            # save logs in lock to avoid a parallel clean_up() removing them
+            if self._browser_logs:
+                log_path = mkdtemp(
+                    prefix=strftime("%Y%m%d-%H%M%S_", localtime()),
+                    suffix="_browser_logs",
+                    dir=self._browser_logs)
+                LOG.debug("saving browser logs to %r", log_path)
+                self._puppet.save_logs(log_path)
+                # only save logs once per launch
+                self._browser_logs = None
 
     @property
     def closed(self):
@@ -99,8 +132,8 @@ class PuppetTarget(Target):
         # check if there has been a crash, hang, etc...
         if not is_healthy or was_timeout:
             if self._puppet.is_running():
-                log.debug("terminating browser...")
-                if was_timeout and "timeout" not in ignored and platform.system() == "Linux":
+                LOG.debug("terminating browser...")
+                if was_timeout and "timeout" not in ignored and system() == "Linux":
                     self._abort_hung_proc()
                     # give the process a moment to start dump
                     self._puppet.wait(timeout=1)
@@ -108,51 +141,49 @@ class PuppetTarget(Target):
         # if something has happened figure out what
         if not is_healthy:
             if self._puppet.reason == FFPuppet.RC_CLOSED:
-                log.info("target.close() was called")
+                LOG.info("target.close() was called")
             elif self._puppet.reason == FFPuppet.RC_EXITED:
-                log.info("Target closed itself")
+                LOG.info("Target closed itself")
             elif (self._puppet.reason == FFPuppet.RC_WORKER
                   and "memory" in ignored
                   and "ffp_worker_memory_usage" in self._puppet.available_logs()):
                 status = self.RESULT_IGNORED
-                log.info("Memory limit exceeded")
+                LOG.info("Memory limit exceeded")
             elif (self._puppet.reason == FFPuppet.RC_WORKER
                   and "log-limit" in ignored
                   and "ffp_worker_log_size" in self._puppet.available_logs()):
                 status = self.RESULT_IGNORED
-                log.info("Log size limit exceeded")
+                LOG.info("Log size limit exceeded")
             else:
-                log.debug("failure detected, ffpuppet return code: %r", self._puppet.reason)
+                LOG.debug("failure detected, ffpuppet return code: %r", self._puppet.reason)
                 status = self.RESULT_FAILURE
         elif was_timeout:
-            log.info("Timeout detected")
+            LOG.info("Timeout detected")
             status = self.RESULT_IGNORED if "timeout" in ignored else self.RESULT_FAILURE
         return status
 
     def dump_coverage(self, timeout=15):
-        # Note: This is not required if the browser is going to close.
-        # SIGTERM will also trigger coverage to be synced out.
         pid = self._puppet.get_pid()
         if pid is None or not self._puppet.is_healthy():
-            log.debug("Skipping coverage dump")
+            LOG.debug("Skipping coverage dump (target is not in a good state)")
             return
         # If at this point, the browser is in a good state, i.e. no crashes
         # or hangs, so signal the browser to dump coverage.
         try:
-            for child in psutil.Process(pid).children(recursive=True):
-                log.debug("Sending SIGUSR1 to %d (child)", child.pid)
-                os.kill(child.pid, signal.SIGUSR1)
-        except (psutil.AccessDenied, psutil.NoSuchProcess):  # pragma: no cover
+            for child in Process(pid).children(recursive=True):
+                LOG.debug("Sending SIGUSR1 to %d (child)", child.pid)
+                kill(child.pid, signal.SIGUSR1)
+        except (AccessDenied, NoSuchProcess):  # pragma: no cover
             pass
-        log.debug("Sending SIGUSR1 to %d (parent)", pid)
-        os.kill(pid, signal.SIGUSR1)
-        start_time = time.time()
+        LOG.debug("Sending SIGUSR1 to %d (parent)", pid)
+        kill(pid, signal.SIGUSR1)
+        start_time = time()
         gcda_found = False
         delay = 0.1
         # wait for processes to write .gcno files
         # this should typically take less than 1 second
         while True:
-            for proc in psutil.process_iter(attrs=["pid", "ppid", "open_files"]):
+            for proc in process_iter(attrs=["pid", "ppid", "open_files"]):
                 # check if proc is the target or child process
                 if pid in (proc.info["pid"], proc.info["ppid"]):
                     if proc.info["open_files"] is None:
@@ -164,31 +195,38 @@ class PuppetTarget(Target):
                         break
             else:
                 gcda_open = None
-            elapsed = time.time() - start_time
+            elapsed = time() - start_time
             if gcda_found:
                 if gcda_open is None:
                     # success
-                    log.debug("gcda dump took %0.2fs", elapsed)
+                    LOG.debug("gcda dump took %0.2fs", elapsed)
                     break
                 if elapsed >= timeout:
                     # timeout failure
-                    log.warning("gcda file open by pid %d after %0.2fs", gcda_open, elapsed)
+                    LOG.warning("gcda file open by pid %d after %0.2fs", gcda_open, elapsed)
+                    kill(gcda_open, signal.SIGABRT)
+                    sleep(1)
+                    self.close()
                     break
-            elif elapsed >= 5:
+                if delay < 1.0:
+                    # increase delay to a maximum of 1 second
+                    delay = min(1.0, delay + 0.1)
+            elif elapsed >= 3:
                 # assume we missed the process writing .gcno files
-                log.warning("No gcda files seen after %0.2fs", elapsed)
+                LOG.warning("No gcda files seen after %0.2fs", elapsed)
                 break
             if not self._puppet.is_healthy():
-                log.warning("Browser failure during dump_coverage()")
+                LOG.warning("Browser failure during dump_coverage()")
                 break
-            time.sleep(delay)
-            if delay < 1.0:
-                # increase delay to a maximum of 1 second
-                delay = min(1.0, delay + 0.1)
+            sleep(delay)
 
     def launch(self, location, env_mod=None):
-        if not self.prefs:
-            raise TargetError("A prefs.js file is required")
+        # GRZ_BROWSER_LOGS is intended to be used to aid in debugging.
+        # when close() is called a copy of the browser logs will be saved
+        # to the directory specified by GRZ_BROWSER_LOGS
+        self._browser_logs = getenv("GRZ_BROWSER_LOGS")
+        if self._browser_logs and not isdir(self._browser_logs):
+            makedirs(self._browser_logs)
         self.rl_countdown = self.rl_reset
         env_mod = dict(env_mod or [])
         # do not allow network connections to non local endpoints
@@ -205,7 +243,7 @@ class PuppetTarget(Target):
                 extension=self.extension,
                 env_mod=env_mod)
         except LaunchError as exc:
-            log.error("FFPuppet LaunchError: %s", str(exc))
+            LOG.error("FFPuppet LaunchError: %s", str(exc))
             self.close()
             if isinstance(exc, BrowserTimeoutError):
                 raise TargetLaunchTimeout(str(exc))
