@@ -372,7 +372,10 @@ def test_sapphire_18(client, tmp_path):
         client.launch("127.0.0.1", serv.port, files_to_serve, in_order=True)
         status, files_served = serv.serve_path(str(root_path), server_map=smap)
     assert status == SERVED_ALL
-    assert len(files_served) == 4
+    assert "test_case.html" in files_served
+    assert str(inc1_path / "included_file1.html") in files_served
+    assert str(inc2_path / "included_file2.html") in files_served
+    assert str(nest_path / "nested_file.html") in files_served
     assert client.wait(timeout=10)
     assert inc1.code == 200
     assert inc2.code == 200
@@ -638,8 +641,11 @@ def test_sapphire_32(mocker):
     assert fake_sock.return_value.listen.call_count == 1
     assert fake_sleep.call_count == 1
 
-def test_main_01(tmp_path):
+def test_main_01(mocker, tmp_path):
     """test Sapphire.main()"""
-    with pytest.raises(SystemExit):
-        Sapphire.main(["missing_path"])
-    Sapphire.main([str(tmp_path)])
+    args = mocker.Mock(
+        path=str(tmp_path),
+        port=4536,
+        remote=False,
+        timeout=None)
+    Sapphire.main(args)
